@@ -19,6 +19,7 @@ import logging
 import os
 from argparse import Namespace
 from copy import deepcopy
+from enum import Enum
 from typing import Any, Callable, Dict, IO, MutableMapping, Optional, Union
 from warnings import warn
 
@@ -29,6 +30,7 @@ from pytorch_lightning.utilities import _OMEGACONF_AVAILABLE, AttributeDict, ran
 from pytorch_lightning.utilities.apply_func import apply_to_collection
 from pytorch_lightning.utilities.cloud_io import get_filesystem
 from pytorch_lightning.utilities.cloud_io import load as pl_load
+from pytorch_lightning.utilities.migration import pl_legacy_patch
 from pytorch_lightning.utilities.parsing import parse_class_init_keys
 
 log = logging.getLogger(__name__)
@@ -117,7 +119,7 @@ class ModelIO:
             MyLightningModule.load_from_checkpoint(
                 PATH,
                 num_layers=128,
-                pretrained_ckpt_path: NEW_PATH,
+                pretrained_ckpt_path=NEW_PATH,
             )
 
             # predict
@@ -125,10 +127,11 @@ class ModelIO:
             pretrained_model.freeze()
             y_hat = pretrained_model(x)
         """
-        if map_location is not None:
-            checkpoint = pl_load(checkpoint_path, map_location=map_location)
-        else:
-            checkpoint = pl_load(checkpoint_path, map_location=lambda storage, loc: storage)
+        with pl_legacy_patch():
+            if map_location is not None:
+                checkpoint = pl_load(checkpoint_path, map_location=map_location)
+            else:
+                checkpoint = pl_load(checkpoint_path, map_location=lambda storage, loc: storage)
 
         if hparams_file is not None:
             extension = hparams_file.split(".")[-1]
@@ -212,31 +215,11 @@ class ModelIO:
 
         return model
 
-    def on_load_checkpoint(self, checkpoint: Dict[str, Any]) -> None:
-        """
-        Do something with the checkpoint.
-        Gives model a chance to load something before ``state_dict`` is restored.
-
-        Args:
-            checkpoint: A dictionary with variables from the checkpoint.
-        """
-
-    def on_save_checkpoint(self, checkpoint: Dict[str, Any]) -> None:
-        """
-        Give the model a chance to add something to the checkpoint.
-        ``state_dict`` is already there.
-
-        Args:
-            checkpoint: A dictionary in which you can save variables to save in a checkpoint.
-                Contents need to be pickleable.
-        """
-
     # -------------------------
     # OPTIONAL HOOKS
     # -------------------------
     def on_hpc_save(self, checkpoint: Dict[str, Any]) -> None:
-        """
-        Hook to do whatever you need right before Slurm manager saves the model.
+        """Hook to do whatever you need right before Slurm manager saves the model.
 
         Args:
             checkpoint: A dictionary in which you can save variables to save in a checkpoint.
@@ -244,8 +227,7 @@ class ModelIO:
         """
 
     def on_hpc_load(self, checkpoint: Dict[str, Any]) -> None:
-        """
-        Hook to do whatever you need right before Slurm manager loads the model.
+        """Hook to do whatever you need right before Slurm manager loads the model.
 
         Args:
             checkpoint: A dictionary with variables from the checkpoint.
@@ -265,8 +247,7 @@ def _convert_loaded_hparams(model_args: dict, hparams_type: Optional[Union[Calla
 
 
 def update_hparams(hparams: dict, updates: dict) -> None:
-    """
-    Overrides hparams with new values
+    """Overrides hparams with new values.
 
     >>> hparams = {'c': 4}
     >>> update_hparams(hparams, {'a': {'b': 2}, 'c': 1})
@@ -279,7 +260,6 @@ def update_hparams(hparams: dict, updates: dict) -> None:
     Args:
         hparams: the original params and also target object
         updates: new params to be used as update
-
     """
     for k, v in updates.items():
         # if missing, add the key
@@ -339,8 +319,8 @@ def load_hparams_from_yaml(config_yaml: str, use_omegaconf: bool = True) -> Dict
 
         Args:
             config_yaml: Path to config yaml file
-            use_omegaconf: If both `OMEGACONF_AVAILABLE` and `use_omegaconf` are True,
-                the hparams will be converted to `DictConfig` if possible
+            use_omegaconf: If omegaconf is available and ``use_omegaconf=True``,
+                the hparams will be converted to ``DictConfig`` if possible.
 
     >>> hparams = Namespace(batch_size=32, learning_rate=0.001, data_root='./any/path/here')
     >>> path_yaml = './testing-hparams.yaml'
@@ -367,11 +347,14 @@ def load_hparams_from_yaml(config_yaml: str, use_omegaconf: bool = True) -> Dict
     return hparams
 
 
-def save_hparams_to_yaml(config_yaml, hparams: Union[dict, Namespace]) -> None:
+def save_hparams_to_yaml(config_yaml, hparams: Union[dict, Namespace], use_omegaconf: bool = True) -> None:
     """
     Args:
         config_yaml: path to new YAML file
         hparams: parameters to be saved
+        use_omegaconf: If omegaconf is available and ``use_omegaconf=True``,
+            the hparams will be converted to ``DictConfig`` if possible.
+
     """
     fs = get_filesystem(config_yaml)
     if not fs.isdir(os.path.dirname(config_yaml)):
@@ -384,7 +367,7 @@ def save_hparams_to_yaml(config_yaml, hparams: Union[dict, Namespace]) -> None:
         hparams = dict(hparams)
 
     # saving with OmegaConf objects
-    if _OMEGACONF_AVAILABLE:
+    if _OMEGACONF_AVAILABLE and use_omegaconf:
         # deepcopy: hparams from user shouldn't be resolved
         hparams = deepcopy(hparams)
         hparams = apply_to_collection(hparams, DictConfig, OmegaConf.to_container, resolve=True)
@@ -402,6 +385,7 @@ def save_hparams_to_yaml(config_yaml, hparams: Union[dict, Namespace]) -> None:
     # drop paramaters which contain some strange datatypes as fsspec
     for k, v in hparams.items():
         try:
+            v = v.name if isinstance(v, Enum) else v
             yaml.dump(v)
         except TypeError:
             warn(f"Skipping '{k}' parameter because it is not possible to safely dump to YAML.")
