@@ -238,18 +238,15 @@ class LightningLite(ABC):
                 )
             sampler = self._get_distributed_sampler(dataloader, **self._strategy.distributed_sampler_kwargs)
 
-        dataloader_kwargs = TrainerDataLoadingMixin._get_dataloader_init_kwargs(dataloader, sampler)
-        try:
-            dataloader = type(dataloader)(**dataloader_kwargs)
-        except TypeError:
-            dataloader_kwargs.pop("dataset")
-            dataloader = type(dataloader)(**dataloader_kwargs)
+        # the dataloader needs to be re-instantiated because we want to update the input arguments (e.g., sampler)
+        dataloader = TrainerDataLoadingMixin._update_dataloader(dataloader, sampler)
+
         # add worker_init_fn for correct seeding in worker processes
         TrainerDataLoadingMixin._auto_add_worker_init_fn(dataloader, self.global_rank)
-        return _LiteDataLoader(
-            dataloader=self._strategy.process_dataloader(dataloader),
-            device=self.device if move_to_device and not isinstance(self._strategy, TPUSpawnPlugin) else None,
-        )
+
+        dataloader = self._strategy.process_dataloader(dataloader)
+        device = self.device if move_to_device and not isinstance(self._strategy, TPUSpawnPlugin) else None
+        return _LiteDataLoader(dataloader=dataloader, device=device)
 
     def backward(self, tensor: Tensor, *args: Any, model: Optional[_LiteModule] = None, **kwargs: Any) -> None:
         """Replaces ``loss.backward()`` in your training loop. Handles precision and automatically for you.
